Common subdirectories: drakma-old/doc and drakma-new/doc
diff -u drakma-old/request.lisp drakma-new/request.lisp
--- drakma-old/request.lisp	2007-04-07 14:21:01.000000000 -0400
+++ drakma-new/request.lisp	2007-06-08 10:49:57.000000000 -0400
@@ -61,39 +61,47 @@
 (defun make-form-data-function (parameters boundary)
   "Creates and returns a closure which can be used as an argument for
 SEND-CONTENT to send PARAMETERS as a `multipart/form-data' request
-body using the boundary BOUNDARY."
+body using the boundary BOUNDARY. 
+If a file is a part of parameters it may be a pathname, a stream or a function 
+which takes one argument, the stream to the server"
   (lambda (stream)
-    (flet ((crlf ()
-             "Sends carriage return and linefeed to STREAM."
-             (write-char #\Return stream)
-             (write-char #\Linefeed stream)))
-      (dolist (name/value parameters)
-        (destructuring-bind (name . value)
-            name/value
-          (when (pathnamep value)
-            (setq value (list value)))
-          (format stream "--~A" boundary)
-          (crlf)
-          (format stream "Content-Disposition: form-data; name=\"~A\"" name)
-          (cond ((stringp value)
-                 (crlf) (crlf)
-                 (format stream "~A" value))
-                ((listp value)
-                 (let* ((pathname (first value))
-                        (filename (or (getf (rest value) :filename)
-                                      (file-namestring pathname)))
-                        (content-type (or (getf (rest value) :content-type)
-                                          "application/octet-stream")))
-                   (format stream "; filename=\"~A\"" filename)
-                   (crlf)
-                   (format stream "Content-Type: ~A" content-type)
-                   (crlf) (crlf)
-                   ;; use SEND-CONTENT to send file as binary data
-                   (send-content pathname stream)))
-                (t (error "Don't know what to do with ~S in multipart/form-data body." value)))
-          (crlf)))
-      (format stream "--~A--" boundary)
-      (crlf))))
+    (let ((original-stream stream)
+	  (stream (if *header-stream* (make-broadcast-stream stream *standard-output*) stream)))
+      (flet ((crlf ()
+	       "Sends carriage return and linefeed to STREAM."
+	       (write-char #\Return stream)
+	       (write-char #\Linefeed stream)))
+	(dolist (name/value parameters)
+	  (destructuring-bind (name . value)
+	      name/value
+	    (when (or (pathnamep value)
+		      (streamp value)
+		      (functionp value))
+	      (setq value (list value)))
+	    (format stream "--~A" boundary)
+	    (crlf)
+	    (format stream "Content-Disposition: form-data; name=\"~A\"" name)
+	    (cond ((stringp value)
+		   (crlf) (crlf)
+		   (format stream "~A" value))
+		  ((listp value)
+		   (let* ((pathname (first value))
+			  (filename (or (if (functionp pathname) "user-closure")
+					(if (streamp pathname) "user-stream")
+					(getf (rest value) :filename)
+					(file-namestring pathname)))
+			  (content-type (or (getf (rest value) :content-type)
+					    "application/octet-stream")))
+		     (format stream "; filename=\"~A\"" filename)
+		     (crlf)
+		     (format stream "Content-Type: ~A" content-type)
+		     (crlf) (crlf)
+		     ;; use SEND-CONTENT to send file as binary data
+		     (send-content pathname original-stream)))
+		  (t (error "Don't know what to do with ~S in multipart/form-data body." value)))
+	    (crlf)))
+	(format stream "--~A--" boundary)
+	(crlf)))))
 
 (defun read-body (stream headers must-close textp)
   "Reads the message body from the HTTP stream STREAM using the
